/*-
 * Copyright (c) 2015 Weongyo Jeong <weongyo@3rocks.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

var g_conf_pac_config_default = `function FindProxyForURL(url, host) {
  host = host.toLowerCase();
  if (isPlainHostName(host) ||
      isInNetEx(host, "127.0.0.0/8") ||
      isInNetEx(host, "10.0.0.0/8") ||
      isInNetEx(host, "172.16.0.0/12") ||
      isInNetEx(host, "192.168.0.0/16") ||
      isInNetEx(host, "fc00::/7") ||
      isInNetEx(host, "fe80::/10") ||
      shExpMatch(url, "http://mudfish.net/*") ||
      shExpMatch(url, "https://mudfish.net/*")) {
        return "DIRECT";
  }
  return "{0}";
}`;

var g_conf = {
  check_current_ip: true,
  refresh_active_tab: false,
  use_secure_connection: false,
  use_ssl_bump: false,
  remember_me: false,
  username: "",
  password: "",
  pac_config: g_conf_pac_config_default
}

function sendAuth() {
  chrome.runtime.sendMessage({
    type: 'SET_AUTH',
    username: g_conf.username,
    password: g_conf.password,
  }, function (response) {
    if (response.error_code != 0) {
      console.log("Failed to send 'SET_AUTH' command.");
    }
  });
}

function sendPacConfig() {
  chrome.runtime.sendMessage({
    type: 'SET_PAC_CONFIG',
    pac_config: g_conf.pac_config
  }, function (response) {
    if (response.error_code != 0) {
      console.log("Failed to send 'SET_PAC_CONFIG' command.");
    }
  });
}

function applyPacConfig() {
  chrome.runtime.sendMessage({
    type: 'GET_STATUS'
  }, function (r1) {
    if (r1.error_code != 0) {
      console.log("[ERROR] Failed to send 'GET_STATUS' command.");
      return;
    }
    if (!r1.connected) {
      console.log("[INFO] It's not connected.");
      $.notify(chrome.i18n.getMessage("i18n_not_connected"), {
        className: "warn",
        position: "bottom left"
      });
      return;
    }
    chrome.runtime.sendMessage({
      type: 'DISCONNECT'
    }, function (r2) {
      if (r2.error_code != 0) {
        console.log("[ERROR] Failed to send 'DISCONNECT' command.");
        return;
      }
      chrome.runtime.sendMessage({
        type: 'CONNECT',
        sid: r1.sid,
        use_secure_connection: r1.use_secure_connection,
        use_ssl_bump: r1.use_ssl_bump
      }, function (r3) {
        if (r2.error_code != 0) {
          console.log("[ERROR] Failed to send 'CONNECT' command.");
          return;
        }
        $.notify(chrome.i18n.getMessage("i18n_applied"), {
          className: "success",
          position: "bottom left"
        });
      });
    });
  });
}

var g_proxy_domain_dump_enabled = false;
var g_domain_dump_timer = null;

function restoreDumpDomainOption() {
  chrome.runtime.sendMessage({
    type: 'GET_STATUS'
  }, function (resp) {
    g_proxy_domain_dump_enabled = resp.domain_dump_enabled;
    if (g_proxy_domain_dump_enabled) {
      $("#options_advanced_domain_dump_play_fa").removeClass("fa-play").addClass("fa-stop");
      $("#options_advanced_domain_dump_play_status").html(chrome.i18n.getMessage("i18n_domain_dump_recording"));
      enableDumpDomain();
    } else {
      $("#options_advanced_domain_dump_play_fa").removeClass("fa-stop").addClass("fa-play");
      $("#options_advanced_domain_dump_play_status").html(chrome.i18n.getMessage("i18n_domain_dump_stopped"));
      disableDumpDomain();
    }
  });
}

function getDumpDomainHostnames() {
  chrome.runtime.sendMessage({
    type: 'GET_DUMP_DOMAIN'
  }, function (response) {
    response.hostnames.sort();
    $("#options_advanced_domain_dump_result").val(response.hostnames.join("\n"));
  });
}

function timerDumpDomain() {
  getDumpDomainHostnames();
  if (g_domain_dump_timer != null) {
    window.clearTimeout(g_domain_dump_timer);
  }
  g_domain_dump_timer = window.setTimeout(timerDumpDomain, 3000);
}

function enableDumpDomain() {
  chrome.runtime.sendMessage({
    type: 'SET_DUMP_DOMAIN',
    enable: true
  }, function (response) {
    g_proxy_domain_dump_enabled = true;
    if (g_domain_dump_timer != null) {
      window.clearTimeout(g_domain_dump_timer);
    }
    g_domain_dump_timer = window.setTimeout(timerDumpDomain, 3000);
    $("#options_advanced_domain_dump_play_fa").removeClass("fa-play").addClass("fa-stop");
    $("#options_advanced_domain_dump_play_status").html(chrome.i18n.getMessage("i18n_domain_dump_recording"));
  });
}

function disableDumpDomain() {
  chrome.runtime.sendMessage({
    type: 'SET_DUMP_DOMAIN',
    enable: false
  }, function (response) {
    g_proxy_domain_dump_enabled = false;
    $("#options_advanced_domain_dump_play_fa").removeClass("fa-stop").addClass("fa-play");
    $("#options_advanced_domain_dump_play_status").html(chrome.i18n.getMessage("i18n_domain_dump_stopped"));
  });
}

function restoreOptions() {
  chrome.storage.sync.get(g_conf, function (items) {
    g_conf = items;
    if (g_conf.check_current_ip) {
      $("#check_current_ip").prop("checked", true);
    } else {
      $("#check_current_ip").removeProp("checked");
    }
    if (g_conf.refresh_active_tab) {
      $("#refresh_active_tab").prop("checked", true);
    } else {
      $("#refresh_active_tab").removeProp("checked");
    }
    if (g_conf.use_secure_connection) {
      $("#use_secure_connection").prop("checked", true);
    } else {
      $("#use_secure_connection").removeProp("checked");
    }
    if (g_conf.use_ssl_bump) {
      $("#use_ssl_bump").prop("checked", true);
    } else {
      $("#use_ssl_bump").removeProp("checked");
    }
    if (g_conf.remember_me) {
      $("#remember_me").prop("checked", true);
      $("#username").removeProp("disabled");
      $("#username").val(g_conf.username);
      $("#password").removeProp("disabled");
      $("#password").val(g_conf.password);
      sendAuth();
    } else {
      $("#remember_me").removeProp("checked");
    }
    $("#pac_config").val(g_conf.pac_config);
  });
  restoreDumpDomainOption();
  getDumpDomainHostnames();
}

function saveOptions() {
  if ($("#check_current_ip").prop("checked") == true) {
    g_conf.check_current_ip = true;
  } else {
    g_conf.check_current_ip = false;
  }
  if ($("#refresh_active_tab").prop("checked") == true) {
    g_conf.refresh_active_tab = true;
  } else {
    g_conf.refresh_active_tab = false;
  }
  if ($("#use_secure_connection").prop("checked") == true) {
    g_conf.use_secure_connection = true;
  } else {
    g_conf.use_secure_connection = false;
  }
  if ($("#use_ssl_bump").prop("checked") == true) {
    g_conf.use_ssl_bump = true;
  } else {
    g_conf.use_ssl_bump = false;
  }
  if ($("#remember_me").prop("checked") == true) {
    g_conf.remember_me = true;
  } else {
    g_conf.remember_me = false;
  }
  g_conf.username = $("#username").val();
  g_conf.password = $("#password").val();
  sendAuth();
  chrome.storage.sync.set(g_conf);
  $.notify(chrome.i18n.getMessage("i18n_saved"), {
    autoHideDelay: 1000,
    className: "success",
    position: "bottom left"
  });
}

var g_codemirror_editor = null;

function changeMenu(menu) {
  $("#a_menu_advanced").removeAttr("class");
  $("#a_menu_general").removeAttr("class");
  $("#mudfish_conf_options_menu_general").css('display', 'none');
  $("#mudfish_conf_options_menu_advanced").css('display', 'none');
  if (menu == "general") {
    $("#a_menu_general").attr("class", "active");
    $("#mudfish_conf_options_menu_general").css('display', '');
  } else if (menu == "advanced") {
    $("#a_menu_advanced").attr("class", "active");
    $("#mudfish_conf_options_menu_advanced").css('display', '');
  }
}

function handleOptions() {
  restoreOptions();
  $("#a_menu_general").click(function () {
    changeMenu("general");
  });
  $("#a_menu_advanced").click(function () {
    changeMenu("advanced");
    if (g_codemirror_editor == null) {
      var pc = document.getElementById("pac_config");
      g_codemirror_editor = CodeMirror.fromTextArea(pc, {
        lineNumbers: true,
        matchBrackets: true
      });
      g_codemirror_editor.on('change', function (cm) {
        g_conf.pac_config = cm.getValue();
        saveOptions();
        sendPacConfig();
      });
    }
  });
  $("#remember_me").click(function () {
    if ($("#remember_me").prop("checked") == true) {
      $("#username").removeProp("disabled");
      $("#password").removeProp("disabled");
    } else {
      $("#username").prop("disabled", true);
      $("#password").prop("disabled", true);
      $("#username").val("");
      $("#password").val("");
    }
    saveOptions();
  });
  $("#check_current_ip").click(function () {
    saveOptions();
  });
  $("#refresh_active_tab").click(function () {
    saveOptions();
  });
  $("#pac_config_apply").click(function () {
    applyPacConfig();
  });
  $("#pac_config_reset").click(function () {
    g_conf.pac_config = g_conf_pac_config_default;
    if (g_codemirror_editor != null) {
      g_codemirror_editor.getDoc().setValue(g_conf.pac_config);
    }
  });
  $("#use_secure_connection").click(function () {
    $("#use_ssl_bump").removeProp("checked");
    saveOptions();
  });
  $("#use_ssl_bump").click(function () {
    $("#use_secure_connection").removeProp("checked");
    saveOptions();
  });
  $("#username").change(function () {
    saveOptions();
  });
  $("#password").change(function () {
    saveOptions();
  });
  $("#options_advanced_domain_dump_play").click(function () {
    if (g_proxy_domain_dump_enabled) {
      disableDumpDomain();
    } else {
      enableDumpDomain();
    }
  });
  $("#options_advanced_domain_dump_upload").click(function () {
    chrome.runtime.sendMessage({
      type: 'GET_DUMP_DOMAIN'
    }, function (response) {
      if (response.hostnames.length <= 0) {
        $.notify(chrome.i18n.getMessage("i18n_domain_dump_no_hostname"), {
          className: "warn",
          position: "bottom left"
        });
        return;
      }
      var username = prompt(chrome.i18n.getMessage("i18n_domain_dump_upload_q1"));
      if (username === null || username.length <= 0) {
        return;
      }
      var reason = prompt(chrome.i18n.getMessage("i18n_domain_dump_upload_q2"));
      if (reason === null || reason.length <= 0) {
        return;
      }
      $.ajax({
        url: "https://crs.mudfish.net/report/webextension_chrome_log.php",
        type: "POST",
        data: {
          reason: reason,
          username: username,
          hostnames: response.hostnames.join(",")
        },
        success: function (data) {
          chrome.runtime.sendMessage({
            type: 'CLEAR_DUMP_DOMAIN'
          }, function (response) {
            $.notify(chrome.i18n.getMessage("i18n_domain_dump_uploaded"), {
              className: "success",
              position: "bottom left"
            });
          });
        },
        error: function (xhr, op, err) {
          $.notify(chrome.i18n.getMessage("i18n_domain_dump_failed_to_upload"), {
            className: "error",
            position: "bottom left"
          });
        }
      });
    });
  });
}

function options_i18n_html(name) {
  var m;

  m = chrome.i18n.getMessage(name);
  $(`[id="${name}"]`).html(m);
}

function options_i18n_attr(name, attr) {
  var m;

  m = chrome.i18n.getMessage(name);
  $(`[id="${name}"]`).attr(attr, m);
}

function options_i18n() {
  var m;

  options_i18n_html("i18n_domain_dump_status");
  options_i18n_html("i18n_domain_dump_upload");
  options_i18n_html("options_menu_advanced");
  options_i18n_html("options_menu_advanced_domain_dump");
  options_i18n_html("options_menu_advanced_domain_dump_help_block");
  options_i18n_html("options_menu_advanced_pac_msg");
  options_i18n_html("options_menu_general");
  options_i18n_html("options_menu_general_auto_login");
  options_i18n_html("options_menu_general_func");
  options_i18n_html("options_menu_general_connection");
  options_i18n_html("options_menu_reset");
  options_i18n_html("options_menu_apply");
  options_i18n_html("options_menu_help");
  options_i18n_html("options_remember_me");
  options_i18n_html("options_username");
  options_i18n_html("options_password");
  options_i18n_html("options_check_ip");
  options_i18n_html("options_refresh_tab");
  options_i18n_html("options_use_ssl");
  options_i18n_html("options_use_ssl_desc");
  options_i18n_html("i18n_options_use_ssl_bump");
  options_i18n_html("i18n_options_use_ssl_bump_desc");
  options_i18n_html("i18n_warning");
  options_i18n_html("options_use_ssl_bump_help_block_desc_1");
  options_i18n_html("options_use_ssl_bump_help_block_desc_2");
  options_i18n_attr("options_homepage", "title");
  options_i18n_attr("a_menu_help", "href");
}

$(document).ready(function () {
  options_i18n();
  handleOptions();
});
