/*-
 * Copyright (c) 2015 Weongyo Jeong <weongyo@3rocks.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

if (!String.prototype.format) {
  String.prototype.format = function () {
    var args = arguments;

    return this.replace(/{(\d+)}/g, function (match, number) {
      return typeof args[number] != 'undefined'
        ? args[number]
        : match
        ;
    });
  };
}

function mud_log(str) {
  console.log(new Date() + " " + str);
}

var g_conf = {
  proxy_pac_config: '                                 \
    function FindProxyForURL(url, host) {                 \
        host = host.toLowerCase();                        \
        if (isPlainHostName(host) ||                      \
            isInNetEx(host, "127.0.0.0/8") ||             \
            isInNetEx(host, "10.0.0.0/8") ||              \
            isInNetEx(host, "172.16.0.0/12") ||           \
            isInNetEx(host, "192.168.0.0/16") ||          \
            isInNetEx(host, "fc00::/7") ||                \
            isInNetEx(host, "fe80::/10") ||               \
            shExpMatch(url, "http://mudfish.net/*") ||    \
            shExpMatch(url, "https://mudfish.net/*")) {   \
                return "DIRECT";                          \
        }                                                 \
        return "{0}";                                     \
    }',
  proxy_staticnodes: null,
  proxy_staticnode_index: -1,
  proxy_staticnode_picked: null,
  proxy_staticnode_sid: -1,
  proxy_staticnode_connected: false,
  proxy_staticnode_use_secure_connection: false,
  proxy_staticnode_use_ssl_bump: false,
  proxy_domain_dump_enabled: false,
  proxy_domain_dump_hostnames: "",
  proxy_username: "",
  proxy_password: ""
};
var g_proxy_domain_dump_new_hostnames = "";

function set_proxy_direct_mode() {
  mud_log("[INFO] setting direct mode via proxy.");
  chrome.proxy.settings.set({ value: { mode: "direct" } }, function () {
  });
}

function domain_dump_listener(requestDetails) {
  var url = new URL(requestDetails.url);
  if (url.protocol == "http:" || url.protocol == "https:") {
    g_proxy_domain_dump_new_hostnames += "|" + url.hostname;
  }
  return { cancel: false };
}

function domain_dump_hostnames_array() {
  var i;

  g_conf.proxy_domain_dump_hostnames += g_proxy_domain_dump_new_hostnames;
  hostnames_map = new Map();
  hostnames = g_conf.proxy_domain_dump_hostnames.split("|");
  for (i = 0; i < hostnames.length; i++) {
    hostnames_map.set(hostnames[i], true);
  }
  var output = Array.from(hostnames_map.keys());
  return output;
}

function domain_dump_enable() {
  g_conf.proxy_domain_dump_enabled = true;
  chrome.webRequest.onBeforeRequest.addListener(
    domain_dump_listener,
    { urls: ["<all_urls>"] }
  );
}

function domain_dump_disable() {
  g_conf.proxy_domain_dump_enabled = false;
  g_conf.proxy_domain_dump_hostnames = "";
  g_proxy_domain_dump_new_hostnames = "";
  chrome.webRequest.onBeforeRequest.removeListener(domain_dump_listener);
}

function send_response(callback, obj) {
  mud_log("[INFO] proxy response: " + JSON.stringify(obj));
  callback(obj);
  chrome.storage.local.set(g_conf);
}

function set_proxy_pac_script_mode() {
  if (g_conf.proxy_staticnode_picked === null) {
    mud_log("[WARN] g_conf.proxy_staticnode_picked is null.");
    return;
  }
  var staticnode = g_conf.proxy_staticnode_picked;
  var use_secure_connection = false;
  var use_ssl_bump = false;
  if (g_conf.proxy_staticnode_use_secure_connection) {
    dst = "HTTPS " + staticnode.hostname + ":18082";
    use_secure_connection = true;
  } else if (g_conf.proxy_staticnode_use_ssl_bump) {
    dst = "PROXY " + staticnode.ip + ":18083";
    use_ssl_bump = true;
  } else {
    dst = "PROXY " + staticnode.ip + ":18080";
  }
  var proxyConfig = {
    mode: "pac_script",
    pacScript: {
      data: g_conf.proxy_pac_config.format(dst)
    }
  };
  mud_log("[INFO] setting pac_script mode via proxy: " + JSON.stringify(proxyConfig));
  chrome.proxy.settings.set({
    value: proxyConfig
  }, function () {
    mud_log("[INFO] completed to set up pac_script.");
  });
}

function handle_message(request, sender, callback) {
  mud_log("[INFO] proxy request: " + JSON.stringify(request));
  if (request.type == "CONNECT") {
    var staticnode = null;
    var i, sid;

    sid = parseInt(request.sid);
    for (i = 0; i < g_conf.proxy_staticnodes.length; i++) {
      if (g_conf.proxy_staticnodes[i].sid == sid) {
        staticnode = g_conf.proxy_staticnodes[i];
        g_conf.proxy_staticnode_picked = staticnode;
        break;
      }
    }
    if (staticnode == null || staticnode == undefined) {
      send_response(callback, {
        error_code: 503,
        error_msg: "No static node found"
      });
      return;
    }
    var use_secure_connection = false;
    var use_ssl_bump = false;
    if (request.use_secure_connection) {
      dst = "HTTPS " + staticnode.hostname + ":18082";
      use_secure_connection = true;
    } else if (request.use_ssl_bump) {
      dst = "PROXY " + staticnode.ip + ":18083";
      use_ssl_bump = true;
    } else {
      dst = "PROXY " + staticnode.ip + ":18080";
    }
    var proxyConfig = {
      mode: "pac_script",
      pacScript: {
        data: g_conf.proxy_pac_config.format(dst)
      }
    };
    mud_log("[INFO] setting pac_script via proxy: " + JSON.stringify(proxyConfig));
    chrome.proxy.settings.set({
      value: proxyConfig
    }, function () {
      g_conf.proxy_staticnode_sid = sid;
      g_conf.proxy_staticnode_connected = true;
      g_conf.proxy_staticnode_use_secure_connection = use_secure_connection;
      g_conf.proxy_staticnode_use_ssl_bump = use_ssl_bump;
      chrome.storage.local.set(g_conf);
    });
    send_response(callback, { error_code: 0 });
  } else if (request.type == "CLEAR_DUMP_DOMAIN") {
    g_conf.proxy_domain_dump_hostnames = "";
    g_proxy_domain_dump_new_hostnames = "";
    send_response(callback, { error_code: 0 });
  } else if (request.type == "DISCONNECT") {
    g_conf.proxy_staticnode_sid = -1;
    g_conf.proxy_staticnode_connected = false;
    g_conf.proxy_staticnode_use_secure_connection = false;
    g_conf.proxy_staticnode_use_ssl_bump = false;
    set_proxy_direct_mode();
    send_response(callback, { error_code: 0 });
  } else if (request.type == "SYNC_STATICNODES") {
    fetch_staticnodes_manual(callback);
    /*
     * This need to let chrome know that this will be async.
     */
    return true;
  } else if (request.type == "GET_STATICNODES") {
    if (g_conf.proxy_staticnodes == null) {
      send_response(callback, { error_code: 503, staticnodes: null });
      return;
    }
    send_response(callback, { staticnodes: g_conf.proxy_staticnodes });
  } else if (request.type == "GET_SERVER") {
    var staticnode = g_conf.proxy_staticnode_picked;

    if (staticnode == undefined || staticnode == null) {
      send_response(callback, { location: "Auto Selection" });
      return;
    }
    send_response(callback, { location: staticnode.location });
  } else if (request.type == "GET_DUMP_DOMAIN") {
    send_response(callback, {
      error_code: 0,
      hostnames: domain_dump_hostnames_array()
    });
  } else if (request.type == "GET_STATUS") {
    send_response(callback, {
      error_code: 0,
      connected: g_conf.proxy_staticnode_connected,
      sid: g_conf.proxy_staticnode_sid,
      use_secure_connection: g_conf.proxy_staticnode_use_secure_connection,
      use_ssl_bump: g_conf.proxy_staticnode_use_ssl_bump,
      domain_dump_enabled: g_conf.proxy_domain_dump_enabled
    });
  } else if (request.type == "SET_AUTH") {
    g_conf.proxy_username = request.username;
    g_conf.proxy_password = request.password;
    send_response(callback, { error_code: 0 });
  } else if (request.type == "SET_DUMP_DOMAIN") {
    if (request.enable) {
      domain_dump_enable();
    } else {
      domain_dump_disable();
    }
    send_response(callback, { error_code: 0 });
  } else if (request.type == "SET_PAC_CONFIG") {
    g_conf.proxy_pac_config = request.pac_config;
    send_response(callback, { error_code: 0 });
  } else {
    mud_log("[ERROR] TODO");
    mud_log(request.type);
    send_response(callback, { error_code: 500 });
  }
}

chrome.runtime.onMessage.addListener(function (request, sender, callback) {
  chrome.storage.local.get(g_conf, function (items) {
    g_conf = items;
    handle_message(request, sender, callback);
  });
  return true;
});

async function fetch_staticnodes_success_callback(resp) {
  let data = await resp.json();
  g_conf.proxy_staticnodes = data.staticnodes;
  chrome.storage.local.set(g_conf);
}

function fetch_staticnodes_error_callback(err) {
  var url;

  mud_log("[ERROR] " + err);

  url = chrome.runtime.getURL('/html/staticnodes.json');
  fetch(url).then(function(resp) {
    fetch_staticnodes_success_callback(resp);
  });
}

function fetch_staticnodes() {
  fetch("https://mudfish.net/api/staticnodes")
    .then(fetch_staticnodes_success_callback)
    .catch(fetch_staticnodes_error_callback);
}

function fetch_staticnodes_manual(callback) {
  fetch("https://mudfish.net/api/staticnodes").then(function(resp) {
    fetch_staticnodes_success_callback(resp);
    send_response(callback, { error_code: 0 });
  }).catch(function (err) {
    fetch_staticnodes_error_callback(err);
    send_response(callback, {
      error_code: 500,
      err_msg: "Failed to sync"
    });
  });
}

(() => {
  mud_log("[INFO] proxy.js started.");
  chrome.storage.local.get(g_conf, function (items) {
    g_conf = items;
    mud_log("[INFO] loaded the configuration: " + JSON.stringify(g_conf));
    if (g_conf.proxy_staticnode_connected) {
      chrome.action.setIcon({
        path: "/images/appIcon/default-icon.png"
      });
      chrome.action.setPopup({
        popup: "/html/popup_disconnect.html"
      });
      set_proxy_pac_script_mode();
    } else {
      chrome.action.setIcon({
        path: "/images/appIcon/default-icon-b.png"
      });
      chrome.action.setPopup({
        popup: "/html/popup_connect.html"
      });
      set_proxy_direct_mode();
    }
    fetch_staticnodes();

    chrome.webRequest.onAuthRequired.addListener(
      function (details, callbackFn) {
        if (details.isProxy == false) {
          callbackFn({ cancel: false });
          return;
        }
        if (g_conf.proxy_username.length == 0 ||
          g_conf.proxy_password.length == 0) {
          callbackFn({ cancel: false });
          return;
        }
        callbackFn({
          authCredentials: { username: g_conf.proxy_username, password: g_conf.proxy_password }
        });
      },
      { urls: ["<all_urls>"] },
      ['asyncBlocking']
    );
  });
})();
